# coding: utf-8

"""   
    template render
    ~~~~~~~~~~~~~~
    
    Some tools around template rendering
    
    :copyleft: 2009-2015 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

from django.template import Context, Template

def render_string_template(template, context):
    """
    render the given template string with the context and return it as a string
    
    >>> x = render_string_template("Foo {{ bar }}!", {"bar": "BAR"})
    >>> x == 'Foo BAR!'
    True
    """
    t = Template(template)
    c = Context(context)
    return t.render(c)





# coding: utf-8

"""
    django-tools test models
    ~~~~~~~~~~~~~~~~~~~~~~~~
        
    :copyleft: 2012 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function


from django.db import models
from django_tools import limit_to_usergroups


class LimitToUsergroupsTestModel(models.Model):
    permit_edit = limit_to_usergroups.UsergroupsModelField()
    permit_view = limit_to_usergroups.UsergroupsModelField()


#!/usr/bin/env python
# coding: utf-8

"""
    Show responses in webbrowser
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Helper functions for displaying test responses in webbrowser.

    :copyleft: 2009-2011 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function


from pprint import pformat
from xml.sax.saxutils import escape
import os
import tempfile
import webbrowser

from django.contrib import messages
from django.views.debug import get_safe_settings

from django_tools.utils.stack_info import get_stack_info


# Bug with Firefox under Ubuntu.
# http://www.python-forum.de/topic-11568.html
#webbrowser._tryorder.insert(0, 'epiphany') # Use Epiphany, if installed.

# Variable to save if the browser is opend in the past.
BROWSER_TRACEBACK_OPENED = False

RESPONSE_INFO_ATTR = (
    "content", "cookies", "request", "status_code", "_headers", "context",
)





def debug_response(response, browser_traceback=True, msg="", display_tb=True):
    """
    Display the response content with a error traceback in a webbrowser.
    TODO: We should delete the temp files after viewing!
    """
    global BROWSER_TRACEBACK_OPENED
    if browser_traceback != True or BROWSER_TRACEBACK_OPENED == True:
        return
    # Save for the next traceback
    BROWSER_TRACEBACK_OPENED = True

    content = response.content.decode("utf-8")
    url = response.request["PATH_INFO"]

    stack_info = get_stack_info(filepath_filter="django_tools")
    stack_info.append(msg)
    if display_tb:
        print()
        print("debug_response:")
        print("-" * 80)
        print("\n".join(stack_info))
        print("-" * 80)

    stack_info = escape("".join(stack_info))

    response_info = "<dl>\n"

    response_info += "\t<dt><h3>template</h3></dt>\n"
    if hasattr(response, "templates") and response.templates:
        try:
            templates = pformat([template.name for template in response.templates])
        except AttributeError:
            templates = "---"
    else:
        templates = "---"
    response_info += "\t<dd><pre>%s</pre></dd>\n" % templates

    response_info += "\t<dt><h3>messages</h3></dt>\n"
    msg = messages.get_messages(response.request)
    if msg:
        msg = "".join(["%s\n" % x for x in msg])
    else:
        msg = "---"
    response_info += "\t<dd><pre>%s</pre></dd>\n" % msg

    for attr in RESPONSE_INFO_ATTR:
        # FIXME: There must be exist a easier way to display the info
        response_info += "\t<dt><h3>%s</h3></dt>\n" % attr
        value = getattr(response, attr, "---")
        value = pformat(value)
        value = escape(value)
        response_info += "\t<dd><pre>%s</pre></dd>\n" % value

    response_info += "\t<dt><h3>settings</h3></dt>\n"
    response_info += "\t<dd><pre>%s</pre></dd>\n" % pformat(get_safe_settings())

    response_info += "</dl>\n"


    if "</body>" in content:
        info = (
            "\n<br /><hr />\n"
            "<h2>Unittest info</h2>\n"
            "<dl>\n"
            "<dt><h3>url:</h3></dt><dd>%s</dd>\n"
            "<dt><h3>traceback:</h3></dt><dd><pre>%s</pre></dd>\n"
            "<dt><h2>response info:</h2></dt>%s\n"
            "</dl>\n"
            "</body>"
        ) % (url, stack_info, response_info)
        content = content.replace("</body>", info)
    else:
        # Not a html page?
        content += "\n<pre>\n"
        content += "-" * 79
        content += (
            "\nUnittest info\n"
            "=============\n"
            "url: %s\n"
            "traceback:\n%s\n</pre>"
            "response info:\n%s\n"
        ) % (url, stack_info, response_info)


    fd, file_path = tempfile.mkstemp(prefix="PyLucid_unittest_", suffix=".html")
    os.write(fd, content.encode("utf-8"))
    os.close(fd)
    url = "file://%s" % file_path
    print("\nDEBUG html page in Browser! (url: %s)" % url)
    try:
        webbrowser.open(url)
    except:
        pass
    #time.sleep(0.5)
    #os.remove(file_path)




# coding: utf-8

"""
    django-tools import helper
    ~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    additional helper to the existing django.utils.importlib
    
    :copyleft: 2012 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function

import logging
import os
from importlib import import_module

if __name__ == "__main__":
    # For doctest only
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


logger = logging.getLogger("DjangoToolsImportLib")


def get_attr_from_string(path, obj_name=""):
    """
    Return a attribute from a module by the given path string.
    
    >>> get_attr_from_string("django_tools.cache.site_cache_middleware.UpdateCacheMiddleware")
    <class 'django_tools.cache.site_cache_middleware.UpdateCacheMiddleware'>
    
    >>> get_attr_from_string("no point", "TEST") # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ...
    ImproperlyConfigured: no point isn't a TEST module
    
    >>> get_attr_from_string("django.core.doesntexists", "Foo Bar")
    Traceback (most recent call last):
    ...
    ImproperlyConfigured: Foo Bar module "django.core" does not define a "doesntexists" class
    
    >>> get_attr_from_string("django_tools.utils.importlib.not_exists", "Import Lib")
    Traceback (most recent call last):
    ...
    ImproperlyConfigured: Import Lib module "django_tools.utils.importlib" does not define a "not_exists" class
    """
    try:
        module_name, class_name = path.rsplit('.', 1)
    except ValueError:
        raise ImproperlyConfigured("%s isn't a %s module" % (path, obj_name))
    try:
        mod = import_module(module_name)
    except ImportError as e:
        raise ImproperlyConfigured('Error importing %s module %s: "%s"' % (obj_name, module_name, e))
    try:
        attr = getattr(mod, class_name)
    except AttributeError:
        raise ImproperlyConfigured('%s module "%s" does not define a "%s" class' % (obj_name, module_name, class_name))

    return attr


def get_class_instance(path, obj_name=""):
    """
    Returns a class instance from a module by the given path string.
    
    >>> get_class_instance("django_tools.cache.site_cache_middleware.UpdateCacheMiddleware") # doctest: +ELLIPSIS
    <django_tools.cache.site_cache_middleware.UpdateCacheMiddleware object at ...>
    """
    class_obj = get_attr_from_string(path, obj_name)
    class_instance = class_obj()
    return class_instance


def get_setting(setting_name):
    """
    return the settings value, create debug log if not set/empty.
    
    >>> get_setting("EMAIL_BACKEND")
    'django.core.mail.backends.smtp.EmailBackend'
    
    >>> get_setting("DOESN'T EXISTS")
    >>> get_setting("STATIC_URL")
    
    >>> import sys, logging
    >>> logger = getLogger("DjangoToolsImportLib")
    >>> logger.addHandler(logging.StreamHandler(sys.stdout))
    >>> logger.setLevel(logging.DEBUG)
    
    >>> get_setting("DOESN'T EXISTS")
    "DOESN'T EXISTS" not in settings defined
    
    >>> get_setting("STATIC_URL")
    settings.STATIC_URL is None or empty
    """
    path = getattr(settings, setting_name, None)
    if path:
        return path
    else:
        if not hasattr(settings, setting_name):
            logger.debug("%r not in settings defined" % setting_name)
        else:
            logger.debug("settings.%s is None or empty" % setting_name)


def get_attr_from_settings(setting_name, obj_name=""):
    """
    returns a attribute from the given settings path string.
    
    >>> get_attr_from_settings("EMAIL_BACKEND", "email backend") # doctest: +ELLIPSIS
    <class 'django.core.mail.backends.smtp.EmailBackend'>
    
    >>> assert settings.STATIC_URL is None
    
    If the settings String doesn't exist or is empty, nothing happen:
    
    >>> get_attr_from_settings("DOESN'T EXISTS", "a test")
    >>> get_attr_from_settings("STATIC_URL", "a test")
    """
    path = get_setting(setting_name)
    if path:
        return get_attr_from_string(path, obj_name)


def get_class_instance_from_settings(setting_name, obj_name=""):
    """
    returns a class instance from the given settings path string.
    
    >>> get_class_instance_from_settings("EMAIL_BACKEND", "email backend") # doctest: +ELLIPSIS
    <django.core.mail.backends.smtp.EmailBackend object at ...>
    
    >>> assert settings.STATIC_URL is None
    
    If the settings String doesn't exist or is empty, nothing happen:
    
    >>> get_class_instance_from_settings("DOESN'T EXISTS", "a test")
    >>> get_class_instance_from_settings("STATIC_URL", "a test")
    """
    path = get_setting(setting_name)
    if path:
        return get_class_instance(path, obj_name)


if __name__ == "__main__":
    import doctest
    print(doctest.testmod())



# coding: utf-8

from __future__ import print_function

print("Use settings:", __file__)

DEBUG = True
TEMPLATE_DEBUG = DEBUG

SECRET_KEY = "Unittests"
ALLOWED_HOSTS = ["*"] # Allow any domain/subdomain

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ":memory:"
    }
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'unique-snowflake'
    }
}

MIDDLEWARE_CLASSES = (
    'django_tools.dynamic_site.middleware.DynamicSiteMiddleware',
    'django_tools.middlewares.ThreadLocal.ThreadLocalMiddleware',
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',

    'django_tools',
    'django_tools.local_sync_cache',
    'django_tools.dynamic_site',
    'django_tools_test_project.django_tools_test_app',
)
SITE_ID = 1

USE_DYNAMIC_SITE_MIDDLEWARE = True

ROOT_URLCONF = 'django_tools_test_project.django_tools_test_app.urls'

PASSWORD_HASHERS = ( # Speedup tests
    'django.contrib.auth.hashers.MD5PasswordHasher',
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(msecs)d %(module)s.%(funcName)s line %(lineno)d: %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'null': {'class': 'logging.NullHandler',},
        'console': {
            'class': 'logging.StreamHandler',
            # 'formatter': 'simple'
            'formatter': 'verbose'
        },
    },
    'loggers': {
        "django_tools": {
            'handlers': [
                'null',
                # 'console'
            ],
            'level': 'DEBUG',
        },
        "django_tools.DynamicSite": {
            'handlers': [
                'null',
                # 'console'
            ],
            'level': 'DEBUG',
        },
    },
}


# coding: utf-8

"""
    Secure JavaScript Login
    ~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2015 by the django-tools team, see AUTHORS for more details.
    :created: by JensDiemer.de
    :license: GNU GPL v3 or above, see LICENSE for more details
"""

from __future__ import unicode_literals, print_function

import os
import doctest
import sys

import django_tools


SKIP_DIRS = (".settings", ".git", "dist", ".egg-info")
SKIP_FILES = ("setup.py", "test.py")


def get_all_doctests(base_path, verbose=False):
    modules = []
    for root, dirs, filelist in os.walk(base_path, followlinks=True):
        for skip_dir in SKIP_DIRS:
            if skip_dir in dirs:
                dirs.remove(skip_dir) # don't visit this directories

        for filename in filelist:
            if not filename.endswith(".py"):
                continue
            if filename in SKIP_FILES:
                continue

            sys.path.insert(0, root)
            try:
                module = __import__(filename[:-3])
            except ImportError as err:
                if verbose:
                    print(
                        "\tDocTest import %s error %s" % (filename, err),
                        file=sys.stderr
                    )
            except Exception as err:
                if verbose:
                    print(
                        "\tDocTest %s error %s" % (filename, err),
                        file=sys.stderr
                    )
            else:
                try:
                    suite = doctest.DocTestSuite(module)
                except ValueError: # has no docstrings
                    continue

                test_count = suite.countTestCases()
                if test_count<1:
                    if verbose:
                        print(
                            "\tNo DocTests in %r" % module.__name__,
                            file=sys.stderr
                        )
                    continue

                if verbose:
                    file_info = module.__file__
                else:
                    file_info = module.__name__
                print(
                    "\t%i DocTests in %r" % (test_count,file_info),
                    file=sys.stderr
                )
                modules.append(module)
            finally:
                del sys.path[0]

    return modules



def load_tests(loader, tests, ignore):
    print("\ncollect DocTests:", file=sys.stderr)
    path = os.path.abspath(os.path.dirname(django_tools.__file__))
    modules = get_all_doctests(
        base_path=path,
        # verbose=True
    )
    # TODO:
    # for module in modules:
    #     suite = doctest.DocTestSuite(module)
    #     suite = unittest.expectedFailure(suite) # FIXME
    #     tests.addTests(suite)
    return tests



# coding: utf-8

"""   
    unittest base
    ~~~~~~~~~~~~~
    
    :copyleft: 2009-2015 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function


import os
import sys
import textwrap

from django.test import TestCase
from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from django.core import management
from django.test import SimpleTestCase
from django.test.html import HTMLParseError, parse_html
from django.utils.encoding import smart_str

from .BrowserDebug import debug_response
import difflib

class BaseUnittestCase(TestCase):
    """
    Extensions to plain Unittest TestCase
    """
    def _dedent(self, txt):
        # Remove any common leading whitespace from every line
        txt = textwrap.dedent(txt)

        # strip whitespace at the end of every line
        txt = "\n".join([line.rstrip() for line in txt.splitlines()])
        txt = txt.strip()
        return txt

    def assertEqual_dedent(self, first, second, msg=None):
        first = self._dedent(first)
        second = self._dedent(second)
        try:
            self.assertEqual(first, second, msg)
        except AssertionError as err:
            # Py2 has a bad error message
            msg = (
                "%s\n"
                " ------------- [first] -------------\n"
                "%s\n"
                " ------------- [second] ------------\n"
                "%s\n"
                " -----------------------------------\n"
            ) % (err, first, second)
            raise AssertionError(msg)

    def assert_is_dir(self, path):
        if not os.path.isdir(path):
            self.fail("Directory %r doesn't exists!" % path)

    def assert_not_is_dir(self, path):
        if os.path.isdir(path):
            self.fail("Directory %r exists, but should not exists!" % path)

    def assert_is_file(self, path):
        if not os.path.isfile(path):
            self.fail("File %r doesn't exists!" % path)

    def assert_not_is_File(self, path):
        if os.path.isfile(path):
            self.fail("File %r exists, but should not exists!" % path)


class BaseTestCase(BaseUnittestCase):
    # Should we open a browser traceback?
    browser_traceback = True

    TEST_USERS = {
        "superuser": {
            "username": "superuser",
            "email": "superuser@example.org",
            "password": "superuser_password",
            "is_staff": True,
            "is_superuser": True,
        },
        "staff": {
            "username": "staff test user",
            "email": "staff_test_user@example.org",
            "password": "staff_test_user_password",
            "is_staff": True,
            "is_superuser": False,
        },
        "normal": {
            "username": "normal test user",
            "email": "normal_test_user@example.org",
            "password": "normal_test_user_password",
            "is_staff": False,
            "is_superuser": False,
        },
    }

    def _pre_setup(self):
        super(BaseTestCase, self)._pre_setup()

    def create_user(self, verbosity, username, password, email, is_staff, is_superuser):
        """
        Create a user and return the instance.
        """
        defaults = {'password':password, 'email':email}
        user, created = User.objects.get_or_create(
            username=username, defaults=defaults
        )
        if not created:
            user.email = email
        user.set_password(password)
        user.is_staff = is_staff
        user.is_superuser = is_superuser
        user.save()
        if verbosity >= 2:
            print("Test user %r created." % user)
        return user
    
    def create_testusers(self, verbosity=2):
        """
        Create all available testusers and UserProfiles
        """
        for userdata in list(self.TEST_USERS.values()):
            self.create_user(verbosity, **userdata)

    def login(self, usertype):
        """
        Login the user defined in self.TEST_USERS
        return User model instance
        """
        test_user = self._get_userdata(usertype)

        count = User.objects.filter(username=test_user["username"]).count()
        self.failIfEqual(count, 0, "You have to call self.create_testusers() first!")
        self.failUnlessEqual(count, 1)

        ok = self.client.login(username=test_user["username"],
                               password=test_user["password"])
        self.failUnless(ok, "Can't login test user '%s'!" % usertype)
        return self._get_user(usertype)

    def add_user_permissions(self, user, permissions):
        """
        add permissions to the given user instance.
        permissions e.g.: ("AppLabel.add_Modelname", "auth.change_user")
        """
        assert isinstance(permissions, (list, tuple))
        for permission in permissions:
            # permission, e.g: blog.add_blogentry
            app_label, permission_codename = permission.split(".", 1)
            model_name = permission_codename.split("_", 1)[1]

            try:
                content_type = ContentType.objects.get(app_label=app_label, model=model_name)
            except ContentType.DoesNotExist:
                etype, evalue, etb = sys.exc_info()
                evalue = etype("Can't get ContentType for app %r and model %r: %s" % (
                    app_label, model_name, evalue
                ))
                raise etype(evalue).with_traceback(etb)


            perm = Permission.objects.get(content_type=content_type, codename=permission_codename)
            user.user_permissions.add(perm)
            user.save()

        self.assertTrue(user.has_perms(permissions))

    def _get_userdata(self, usertype):
        """ return userdata from self.TEST_USERS for the given usertype """
        try:
            return self.TEST_USERS[usertype]
        except KeyError as err:
            etype, evalue, etb = sys.exc_info()
            evalue = etype(
                "Wrong usetype %s! Existing usertypes are: %s" % (err, ", ".join(list(self.TEST_USERS.keys())))
            )
            raise etype(evalue).with_traceback(etb)

    def _get_user(self, usertype):
        """ return User model instance for the given usertype"""
        test_user = self._get_userdata(usertype)
        return User.objects.get(username=test_user["username"])

    def _create_testusers(self):
        """ Create all available testusers. """
        def create_user(username, password, email, is_staff, is_superuser):
            """
            Create a user and return the instance.
            """
            defaults = {'password':password, 'email':email}
            user, created = User.objects.get_or_create(
                username=username, defaults=defaults
            )
            if not created:
                user.email = email
            user.set_password(password)
            user.is_staff = is_staff
            user.is_superuser = is_superuser
            user.save()
        for usertype, userdata in self.TEST_USERS.items():
            create_user(**userdata)

    def raise_browser_traceback(self, response, msg):
        debug_response(
            response, self.browser_traceback, msg, display_tb=False
        )
        msg += " (url: %r)" % response.request.get("PATH_INFO", "???")
        raise self.failureException(msg)

    def assertStatusCode(self, response, excepted_code=200):
        """
        assert response status code, if wrong, do a browser traceback.
        """
        if response.status_code == excepted_code:
            return # Status code is ok.
        msg = "assertStatusCode error: %r != %r" % (response.status_code, excepted_code)
        self.raise_browser_traceback(response, msg)

    # def _assert_and_parse_html(self, html, user_msg, msg):
    #     """
    #     convert a html snippet into a DOM tree.
    #     raise error if snippet is no valid html.
    #     """
    #     try:
    #         return parse_html(html)
    #     except HTMLParseError as e:
    #         self.fail("html code is not valid: %s - code: %r" % (e, html))
    #
    # def _assert_and_parse_html_response(self, response):
    #     """
    #     convert html response content into a DOM tree.
    #     raise browser traceback, if content is no valid html.
    #     """
    #     try:
    #         return parse_html(response.content)
    #     except HTMLParseError as e:
    #         self.raise_browser_traceback(response, "Response's content is no valid html: %s" % e)

    def assertDOM(self, response, must_contain=(), must_not_contain=(), use_browser_traceback=True):
        """
        Asserts that html response contains 'must_contain' nodes, but no
        nodes from must_not_contain.      
        """
        for txt in must_contain:
            try:
                self.assertContains(response, txt, html=True)
            except AssertionError as err:
                if use_browser_traceback:
                    self.raise_browser_traceback(response, err)
                raise

        for txt in must_not_contain:
            try:
                self.assertNotContains(response, txt, html=True)
            except AssertionError as err:
                if use_browser_traceback:
                    self.raise_browser_traceback(response, err)
                raise

    def assertResponse(self, response,
            must_contain=None, must_not_contain=None,
            status_code=200, html=False,
            browser_traceback=True):
        """
        Check the content of the response
        must_contain - a list with string how must be exists in the response.
        must_not_contain - a list of string how should not exists.
        """
        if must_contain is not None:
            for must_contain_snippet in must_contain:
                try:
                    self.assertContains(response, must_contain_snippet,
                        status_code=status_code, html=html
                    )
                except AssertionError as err:
                    if browser_traceback:
                        msg = "Text not in response: '%s': %s" % (
                            must_contain_snippet, err
                        )
                        debug_response(
                            response, self.browser_traceback, msg, display_tb=True
                        )
                    raise

        if must_not_contain is not None:
            for must_not_contain_snippet in must_not_contain:
                try:
                    self.assertNotContains(response, must_not_contain_snippet,
                        status_code=status_code, html=html
                    )
                except AssertionError as err:
                    if browser_traceback:
                        msg = "Text should not be in response: '%s': %s" % (
                            must_not_contain_snippet, err
                        )
                        debug_response(
                            response, self.browser_traceback, msg, display_tb=True
                        )
                    raise


def direct_run(raw_filename):
    """
    Run a test direct from a unittest file.
    A unittest file should add something like this:
    
    if __name__ == "__main__":
        # Run this unittest directly
        direct_run(__file__)
    """
    appname = os.path.splitext(os.path.basename(raw_filename))[0]
    print("direct run %r" % appname)
    management.call_command('test', appname)




# coding: utf-8

"""
    Dynamic SITE ID unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    run only these tests:
        .../django-tools $ ./runtests.sh tests.test_dynamic_site

    To see debug output: enable LOGGING in:
        .../django-tools/django_tools_test_project/test_settings.py
    
    :copyleft: 2012-2015 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import absolute_import, division, print_function


import unittest

from django.conf import settings
from django.contrib.sites.models import Site
from django.test.utils import override_settings
from django.utils import log
from django.utils.encoding import force_bytes, force_str

from django_tools.unittest_utils.BrowserDebug import debug_response
from django_tools.dynamic_site.models import SiteAlias
from django_tools.unittest_utils.unittest_base import BaseTestCase


class FakeResponse(object):
    _charset = "utf-8"
    def __init__(self, content, status_code=200):
        self.content = content
        self.status_code = status_code


@override_settings(DEBUG = True)
class DynamicSiteTest(BaseTestCase):
    def setUp(self):
        self.assertTrue(settings.DEBUG, "Must be true to skip hostname validation!")

        Site.objects.all().delete()
        self.site1 = Site.objects.create(pk=1, domain="domain_one.tld", name="site 1")
        self.assertEqual(self.site1.pk, 1)
        self.site2 = Site.objects.create(pk=2, domain="domain_two.tld", name="site 2")
        self.assertEqual(self.site2.pk, 2)

        self.alias1 = SiteAlias.objects.create(site=self.site1, alias="domain_one_alias_one.tld", regex=False)
        self.alias2 = SiteAlias.objects.create(site=self.site1, alias="domain_one_alias_two.tld", regex=False)
        self.alias3 = SiteAlias.objects.create(site=self.site2, alias="www.domain_one.tld", regex=False)
        self.alias4 = SiteAlias.objects.create(site=self.site2, alias=r"^(.*?\.domain_two\.tld)$", regex=True)

    def test_fallback(self):
        response = self.client.get("/display_site/") # request with 'testserver'
        self.assertEqual(response.content.decode("utf-8"),
            'ID from settings: 1 - id from get_current(): 1'
        )

    def test_no_alias1(self):
        response = self.client.get("/display_site/", HTTP_HOST="domain_one.tld")
        self.assertEqual(response.content.decode("utf-8"),
            'ID from settings: 1 - id from get_current(): 1'
        )

    def test_no_alias2(self):
        response = self.client.get("/display_site/", HTTP_HOST="domain_two.tld")
        self.assertEqual(response.content.decode("utf-8"),
            'ID from settings: 2 - id from get_current(): 2'
        )

    def test_string_alias1(self):
        response = self.client.get("/display_site/", HTTP_HOST="domain_one_alias_one.tld")
        self.assertEqual(response.content.decode("utf-8"),
            'ID from settings: 1 - id from get_current(): 1'
        )

    def test_string_alias2(self):
        response = self.client.get("/display_site/", HTTP_HOST="domain_one_alias_two.tld")
        self.assertEqual(response.content.decode("utf-8"),
            'ID from settings: 1 - id from get_current(): 1'
        )

    def test_regex_alias1(self):
        response = self.client.get("/display_site/", HTTP_HOST="foo.domain_two.tld")
        self.assertEqual(response.content.decode("utf-8"),
            'ID from settings: 2 - id from get_current(): 2'
        )

    def test_regex_alias2(self):
        response = self.client.get("/display_site/", HTTP_HOST="foo.bar.domain_two.tld")
        self.assertEqual(response.content.decode("utf-8"),
            'ID from settings: 2 - id from get_current(): 2'
        )

    def test_change(self):
        response = self.client.get("/display_site/", HTTP_HOST="www.domain_one.tld")
        self.assertEqual(response.content.decode("utf-8"),
            'ID from settings: 2 - id from get_current(): 2'
        )

        self.alias3.site = self.site1
        self.alias3.save() # Should clean all site caches

        response = self.client.get("/display_site/", HTTP_HOST="www.domain_one.tld")
        self.assertEqual(response.content.decode("utf-8"),
            'ID from settings: 1 - id from get_current(): 1'
        )

    def test_create(self):
        host = "not_exist_yet.tld"

        # Fallback to default:
        response = self.client.get("/display_site/", HTTP_HOST=host)
        self.assertEqual(response.content.decode("utf-8"),
            'ID from settings: 1 - id from get_current(): 1'
        )

        # Should clean all site caches:
        SiteAlias.objects.create(site=self.site2, alias=host, regex=False)

        response = self.client.get("/display_site/", HTTP_HOST=host)
        self.assertEqual(response.content.decode("utf-8"),
            'ID from settings: 2 - id from get_current(): 2'
        )




# coding: utf-8

"""
    utilities for settings.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2015 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from __future__ import absolute_import, division, print_function

from fnmatch import fnmatch


class InternalIps(list):
    """
    Allows you to use Unix shell-style wildcards of IP addresses in your INTERNAL_IPS.
    Used the fnmatch module.

    settings.py e.g.:
    --------------------------------------------------------------------------
    from django_tools.settings_utils import InternalIps

    INTERNAL_IPS = InternalIps(["127.0.0.1", "::1", "192.168.*.*", "10.0.*.*"])
    --------------------------------------------------------------------------

    borrowed from https://djangosnippets.org/snippets/1380/
    """
    def __contains__(self, key):
        for ip in self:
            if fnmatch(key, ip): return True
        return False




import os
import unittest
import sys

import django
from django.contrib.auth.models import User
from django.utils import six
from django_tools.unittest_utils.print_sql import PrintQueries
from django_tools.unittest_utils.stdout_redirect import StdoutStderrBuffer
from django_tools.unittest_utils.tempdir import TempDir
from django_tools.unittest_utils.unittest_base import BaseUnittestCase, BaseTestCase


class TestBaseUnittestCase(BaseUnittestCase):
    def test_dedent(self):
        """
        Test for BaseUnittestCase._dedent()
        """
        out1 = self._dedent("""
            one line
            line two""")
        self.assertEqual(out1, "one line\nline two")

        out2 = self._dedent("""
            one line
            line two
        """)
        self.assertEqual(out2, "one line\nline two")

        out3 = self._dedent("""
            one line

            line two
        """)
        self.assertEqual(out3, "one line\n\nline two")

        out4 = self._dedent("""
            one line
                line two

        """)
        self.assertEqual(out4, "one line\n    line two")

        # removing whitespace and the end
        self.assertEqual(self._dedent("\n  111  \n  222"), "111\n222")

        out5 = self._dedent("""
            one line
                line two
            dritte Zeile
        """)
        self.assertEqual(out5, "one line\n    line two\ndritte Zeile")

    def test_assert_is_dir(self):
        existing_path = os.path.dirname(__file__)
        self.assert_is_dir(existing_path)
        try:
            self.assert_is_dir("foobar_dir")
        except AssertionError as err:
            self.assertEqual(six.text_type(err), "Directory 'foobar_dir' doesn't exists!")

    def test_assert_not_is_dir(self):
        self.assert_not_is_dir("foobar_dir")
        existing_path = os.path.dirname(__file__)
        try:
            self.assert_not_is_dir(existing_path)
        except AssertionError as err:
            self.assertEqual(six.text_type(err), "Directory '%s' exists, but should not exists!" % existing_path)


    def test_assert_is_file(self):
        self.assert_is_file(__file__)
        try:
            self.assert_is_file("foobar_file.txt")
        except AssertionError as err:
            self.assertEqual(six.text_type(err), "File 'foobar_file.txt' doesn't exists!")

    def test_assert_not_is_File(self):
        self.assert_not_is_File("foobar_file.txt")
        try:
            self.assert_not_is_File(__file__)
        except AssertionError as err:
            self.assertEqual(six.text_type(err), "File '%s' exists, but should not exists!" % __file__)


class TestTempDir(BaseUnittestCase):

    def testTempDir(self):
        """ test TempDir() """
        with TempDir(prefix="foo_") as tempfolder:
            self.assert_is_dir(tempfolder)
            self.assertIn(os.sep + "foo_", tempfolder)

            test_filepath = os.path.join(tempfolder, "bar")
            open(test_filepath, "w").close()
            self.assert_is_file(test_filepath)

        # cleaned?
        self.assert_not_is_dir(tempfolder)
        self.assert_not_is_File(test_filepath)


class TestStdoutStderrBuffer(BaseUnittestCase):
    def test_text_type(self):
        with StdoutStderrBuffer() as buffer:
            print(six.text_type("print text_type"))
            sys.stdout.write(six.text_type("stdout.write text_type\n"))
            sys.stderr.write(six.text_type("stderr.write text_type"))
        self.assertEqual_dedent(buffer.get_output(), """
            print text_type
            stdout.write text_type
            stderr.write text_type
        """)

    def test_binary_type(self):
        if six.PY2:
            with StdoutStderrBuffer() as buffer:
                print("print str")
                sys.stdout.write("stdout.write str\n")
                sys.stderr.write("stderr.write str")
            self.assertEqual_dedent(buffer.get_output(), """
                print str
                stdout.write str
                stderr.write str
            """)
        elif six.PY3:
            # The print function will use repr
            with StdoutStderrBuffer() as buffer:
                print(b"print binary_type")
                sys.stdout.write(b"stdout.write binary_type\n")
                sys.stderr.write(b"stderr.write binary_type")
            self.assertEqual_dedent(buffer.get_output(), """
                b'print binary_type'
                stdout.write binary_type
                stderr.write binary_type
            """)
        else:
            self.fail()


class TestPrintSQL(BaseTestCase):
    def test(self):
        with StdoutStderrBuffer() as buffer:
            with PrintQueries("Create object"):
                User.objects.all().count()

        output = buffer.get_output()
        # print(output)

        self.assertIn("*** Create object ***", output)
        # FIXME: Will fail if not SQLite/MySQL is used?!?
        if django.VERSION < (1, 9):
            self.assertIn("1 - QUERY = 'SELECT COUNT(", output)
        else:
            self.assertIn("1 - SELECT COUNT(", output)
        self.assertIn('FROM "auth_user"', output)



# encoding: utf-8


import unittest

from django.utils.six import PY2, binary_type

from django_tools.utils import http



class HttpTests(unittest.TestCase):
    def test_http_request(self):
        r = http.HttpRequest("http://www.google.com", timeout=3, threadunsafe_workaround=True)
        response = r.get_response()

        # print("-"*79)
        # print("repr(request_header):")
        # print(repr(response.request_header))
        # print("-"*79)
        # print("response.info():")
        # print(response.info())
        # print("-"*79)

        if PY2:
            self.assertIn("Host: www.google.", response.request_header)
        else:
            self.assertIn(b"Host: www.google.", response.request_header)

        response_info = response.info()
        self.assertEqual(response_info.get("Server"), "gws")
        self.assertEqual(response_info["content-type"], "text/html; charset=ISO-8859-1")

        self.assertEqual(response.getcode(), 200)

        content = r.get_unicode()
        self.assertIn("google.com", content)
        self.assertIn("https", content)

    @unittest.skipIf(not http.HTTPS_SUPPORT, "no https support :(")
    def test_https_request(self):
        r = http.HttpRequest("https://www.google.com", timeout=3, threadunsafe_workaround=True)
        response = r.get_response()
        url = response.geturl()
        self.assertIn("https://www.google.", url)
